#ifndef __DIALDIRECTORY_H__
#define __DIALDIRECTORY_H__

#include <qvaluevector.h>
#include <qdatetime.h>
#include <qthread.h>
#include "design/dialdirectoryui.h"
#include "design/dialingui.h"

class DialerExt;

class DialerThread: public QThread {
public:
  bool dialingDone, connected;
  DialerExt *de;

  DialerThread( const char *phone0, int fd, QObject *eventReceiver );
  ~DialerThread();
  void cancelDialing();
  void redial();
  void increaseTimeout();
  void decreaseTimeout();
protected:
  char *phone;
  QString dialingType;
  int fd;
  QObject *eventReceiver;

  virtual void run();
};

class DialEntry {
public:
  QString name, phone, notes, dlSuffix;
  int calls;
  QDate lastLogin;
  bool iemsiLogin;
  QString iemsiLoginName, iemsiPassword;
};

class DirLVItem: public QListViewItem {
public:
  int listPos;

  DirLVItem( int listPos0, QListView *lv, QListViewItem *after,
	  	QString s1, QString s2, QString s3, QString s4 ):
  	QListViewItem( lv, after, s1, s2, s3, s4 ), listPos(listPos0) {}
};

class DialDirectory: public DialDirectoryUI {
  Q_OBJECT
public:
  DialDirectory( int fd, QWidget *parent );
  virtual ~DialDirectory();
  DialEntry *connectedTo();
protected:
  DialerThread *dialer;
  int fd;
  int connectIdx;

  QValueVector<DialEntry> dialList;
  bool dialListModified;

  void loadList();
  void saveList();
  void keyPressEvent( QKeyEvent *e );
protected slots:
  void doDialing();
  void redial();
  void increaseTimeout();
  void decreaseTimeout();
  void editEntry();
  void addEntry();
  void removeEntry();
  void updateNotesDisplay( QListViewItem *it );
signals:
  void grabSerialLine();
  void releaseSerialLine();
};

#endif // __DIALDIRECTORY_H__
