#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/time.h>
#include "dialer.h"

Dialer::Dialer( int fd0 ): fd( fd0 ),
	dialingPause(10), dialingTimeout(50) {
  pipe( cmdPipe );
}

Dialer::~Dialer() {
  close( cmdPipe[0] );
  close( cmdPipe[1] );
}

int Dialer::getLine( char *buf, int len, int timeout ) {
  int idx, state, rc, maxfd;
  char c;
  fd_set rds;
  timeval tv, start_tv, cur_tv;

  if ( len < 1 ) return 0;

  idx = 0;
  state = 0;
  gettimeofday( &start_tv, 0 );
  for (;;) {
	FD_ZERO( &rds );
	FD_SET( fd, &rds );
	FD_SET( cmdPipe[0], &rds );
	if ( fd > cmdPipe[0] ) maxfd = fd; else maxfd = cmdPipe[0];

	tv.tv_sec = 1;
	tv.tv_usec = 0;

  	rc = select( maxfd+1, &rds, 0, 0, &tv );
	if ( rc == -1 ) return Error;

	gettimeofday( &cur_tv, 0 );
	if ( cur_tv.tv_sec - start_tv.tv_sec >= timeout )
		return Timeout;

	if ( rc == 0 ) continue;

	if ( FD_ISSET( cmdPipe[0], &rds ) ) {
		rc = read( cmdPipe[0], &c, 1 );
		if ( rc != 1 ) continue;
		if ( c == 1 ) return Quiting;
		else if ( c == 2 ) return Redialing;
		else if ( c == 3 ) {
			timeout++;
			continue;
		}
		else if ( c == 4 ) {
			timeout--;
			continue;
		}
	}
	if ( !FD_ISSET( fd, &rds ) ) return Error;

  	rc = read( fd, &c, 1 );
	if ( rc != 1 ) return Error;

	switch ( state ) {
	case 0:
		if ( c != '\r' && c != '\n' ) {
			buf[idx++] = c;
			state = 1;
		}
		break;
	case 1:
		if ( c == '\r' ) break;
		if ( c == '\n' ) goto done;
		buf[idx++] = c;
		break;
	}
  }

done:
  buf[idx] = 0;

  return 0;
}

int Dialer::doSleep( int timeout ) {
  fd_set rfd;
  timeval tv, tv_cur, tv_start;
  char c;
  int rc;

  gettimeofday( &tv_start, 0 );
  for (;;) {
	FD_ZERO( &rfd );
	FD_SET( cmdPipe[0], &rfd );

	tv.tv_sec = 1;
	tv.tv_usec = 0;

	rc = select( cmdPipe[0]+1, &rfd, 0, 0, &tv );
	if ( rc < 0 ) return Error;

	gettimeofday( &tv_cur, 0 );
	if ( tv_cur.tv_sec - tv_start.tv_sec >= timeout ) break;

	if ( FD_ISSET( cmdPipe[0], &rfd ) ) {
		read( cmdPipe[0], &c, 1 );
		if ( c == 1 ) return Quiting;
	}
  }

  return 0;
}

bool Dialer::dial( char *phone ) {
  char c, cmd[128], reply[128];
  int i, rc, cmdlen, timeout, pauseTimeout;
  timeval tv_start, tv_cur;
  bool redialing, quiting;

  puts( "dialer started" );

  sprintf( cmd, "atd%s%s\r", dialingType, phone );
  cmdlen = strlen( cmd );

  for(;;) {
  	redialing = false;
	quiting = false;

  	dialing();

	gettimeofday( &tv_start, 0 );
	write( fd, cmd, cmdlen );
	
	rc = getLine( reply, sizeof(reply), dialingTimeout );
	if ( rc == Timeout ) goto next_try;
	if ( rc == Redialing ) {
		redialing = true;
		cancellingDial();
		goto next_try;
	}
	if ( rc == Error || rc == Quiting ) {
		quiting = true;
		goto next_try;
	}
	
	printf( "reply: %s\n", reply );
	if ( !strncmp( cmd, reply, cmdlen-1 ) ) {
		gettimeofday( &tv_cur, 0 );

		rc = getLine( reply, sizeof reply,
			dialingTimeout - (tv_cur.tv_sec - tv_start.tv_sec) );
		if ( rc == Timeout ) goto next_try;
		if ( rc == Redialing ) {
			redialing = true;
			cancellingDial();
			goto next_try;
		}
		if ( rc == Error || rc == Quiting ) {
			quiting = true;
			goto next_try;
		}
		
		printf( "reply2: %s\n", reply );
		if ( !strncmp( reply, "CONNECT", 7 ) ) {
  			puts( "done, connected" );
			return true;
		}
	}

next_try:
	c = '\r';
	write( fd, &c, 1 );

	if ( quiting ) break;
	
	if ( redialing )
		pauseTimeout = 1;
	else {
		pauseTimeout = dialingPause;
		sleeping();
	}

	gettimeofday( &tv_start, 0 );
	for (;;) {
		gettimeofday( &tv_cur, 0 );
		timeout = pauseTimeout - (tv_cur.tv_sec - tv_start.tv_sec);
		if ( timeout < 0 ) break;

		rc = getLine( reply, sizeof(reply), timeout );
		if ( rc == Error || rc == Quiting ) return false;
		if ( rc == Redialing ) break;
	}
  }

  puts( "done, not connected" );

  return false;
}
