#include <qlabel.h>
#include <qpushbutton.h>
#include <qtimer.h>
#include "dialerext.h"
#include "moc_dialerdialog.h"

DialerDialog::DialerDialog( QString name, QString phone, QWidget *parent ):
	DialingUI( parent, "DialingUI", true ), dialTries(0) {
  tryL->setText( "1" );
  nameL->setText( name );
  phoneL->setText( phone );
  setCaption( QString("Dialing ") + name );
  show();

  connect( redialB, SIGNAL(clicked()), SIGNAL(redialRequested()) );
 
  QTimer *progressTimer = new QTimer( this );
  connect( progressTimer, SIGNAL(timeout()), SLOT(updateProgressInfo()) );
  progressTimer->start( 500 );
}

void DialerDialog::updateProgressInfo() {
  int timeLeft = dialerEventStart + dialerEventTimeout - time(0);
  QString status;

  if ( timeLeft < 0 ) timeLeft = 0;
 
  if ( dialing ) {
  	if ( !timeLeft )
		status = "Dialing...";
	else
  		status = QString("Dialing... (%1 seconds left)").arg( timeLeft );
	statusL->setText( status );
  }
  else if ( sleeping ) {
  	if ( !timeLeft )
		status = "Pause...";
	else
	  	status = QString("Pause... (%1 seconds left)").arg( timeLeft );
	statusL->setText( status );
  }
}

void DialerDialog::keyPressEvent( QKeyEvent *ev ) {
  if ( ev->key() == Key_Plus ) {
	increaseTimeout();
	dialerEventTimeout++;
	updateProgressInfo();
  }
  else if ( ev->key() == Key_Minus ) {
	decreaseTimeout();
	dialerEventTimeout--;
	updateProgressInfo();
  }
  else
	DialingUI::keyPressEvent( ev );
}

void DialerDialog::customEvent( QCustomEvent *event0 ) {
  DialerEvent *e;
 
  switch ( event0->type() ) {
  case DE_DIALING:
  	sleeping = false;
  	dialing = true;
	dialTries++;
	tryL->setText( QString::number(dialTries) );
  	break;
  case DE_SLEEPING:
  	dialing = false;
  	sleeping = true;
  	break;
  case DE_WAIT:
  	dialing = false;
	sleeping = false;
	statusL->setText("Wait...");
	return;
  default:
  	return;
  }

  e = (DialerEvent*)event0;
  dialerEventStart = time(0);
  dialerEventTimeout = e->timeout;
  updateProgressInfo();
}
