#include <qthread.h>
#include <qlabel.h>
#include <sys/ioctl.h>
#include "moc_statusline.h"

StatusLine::StatusLine( QWidget *parent ): StatusLineUI( parent ),
	updateTimer(0), fd(-1), prev_stat(0) {
}

void StatusLine::attachLine( SerialLine *line ) {
  if ( !updateTimer ) {
	updateTimer = new QTimer( this );
	connect( updateTimer, SIGNAL(timeout()), this, SLOT(updateStatus()) );
	updateStatus();
	updateTimer->start( 500 );
  }
  fd = line->fd();
}

void StatusLine::disable() {
  delete updateTimer;
  updateTimer = 0;

  onlineL->setText( "Offline" );
  dcdL->setBackgroundColor( backgroundColor() );
  riL ->setBackgroundColor( backgroundColor() );
  dsrL->setBackgroundColor( backgroundColor() );
  ctsL->setBackgroundColor( backgroundColor() );
}

void StatusLine::updateStatus() {
  int flags, rc, status;

  rc = ioctl( fd, TIOCMGET, &flags );
  if ( rc == -1 ) return;

  status = prev_stat ^ flags;

  if ( status & TIOCM_CD ) {
	if ( flags & TIOCM_CD ) {
		onSince = time(0);
		dcdL->setBackgroundColor( red );
	}
	else {
		onlineL->setText( "Offline" );
		dcdL->setBackgroundColor( backgroundColor() );

		emit cdDropped();
	}
  }

  if ( status & TIOCM_RI ) {
	if ( flags & TIOCM_RI )
		riL->setBackgroundColor( red );
	else
		riL->setBackgroundColor( backgroundColor() );
  }

  if ( status & TIOCM_DSR ) {
	if ( flags & TIOCM_DSR )
		dsrL->setBackgroundColor( red );
	else
		dsrL->setBackgroundColor( backgroundColor() );
  }

  if ( status & TIOCM_CTS ) {
	if ( flags & TIOCM_CTS )
		ctsL->setBackgroundColor( red );
	else
		ctsL->setBackgroundColor( backgroundColor() );
  }

  prev_stat = flags;

  if ( flags & TIOCM_CD ) {
	int delta = time(0) - onSince;
	onlineL->setText( QString().sprintf( "Online %d:%02d",
		 delta / 60, delta % 60 ) );
  }
}
