#include <qcolor.h>
#include <qwidget.h>
#include <qsocketnotifier.h>
#include "serialline.h"
#include "dialdirectory.h"
#include "transferdialog.h"
#include "design/transferui.h"

class Cell {
public:
  QChar c;
  int fg, bg;
};

enum {
  ANSI_ESC,
  ANSI_GET_NUM1,
  ANSI_GET_NUM2,
  ANSI_TEXT
};

enum {
  IEMSI_WAIT_IRQ,
  IEMSI_WAIT_ISI,
  IEMSI_COLLECT_ISI
};

enum XferProto {
  XferXmodem,
  XferXmodem1k,
  XferYmodem,
  XferZmodem
};

class StatusLine;
class XFerThread;
class ScrollbackBuffer;

class TermEmu: public QWidget {
  Q_OBJECT
public:
  TermEmu( QWidget *parent, const QFont& fnt, int xsize0 = 80, int ysize0 = 25 );
  virtual ~TermEmu();

  void addStr( const unsigned char *str, int len );
  void attachLine( SerialLine *line );
  void disconnectLine();
  void setStatusLine( StatusLine *statLine0 ) { statLine = statLine0; }
  void saveSessionData( const QString& fnameTempl );
  void restoreSessionData( const QString& fnameTempl );
protected:
  enum {
	LinesSelection,
	ShortLinesSelection,
	RectSelection,
  } selectionType;

  QFont fnt;
  QTextCodec *codec;
  int xsize, ysize;
  int cellw, cellh;
  Cell *data;
  int cursor;
  bool cursor_on;
  bool lineGrabbed;
  int selectionStart, selectionEnd;
  bool hasSelection, selecting;
  ScrollbackBuffer *sbb;
  int scrollbackLines;

  int num[10], num_idx;
  int ansi_state;
  QColor col_pal[16];
  int fgcolor, bgcolor, curs_color;
  bool bright_color, reverse_color;
  int saved_pos;

  StatusLine *statLine;

  unsigned char zmBuf[6];
  QString dlPreffix;
  XferProto xferProto = XferZmodem;
  bool transferSkipRequested, transferStopRequested;

  DialEntry connInfo;
  bool connInfoValid;

  unsigned char iemsiBuf[15];
  char *iemsiData;
  int iemsiState, iemsiDataLen, iemsiDataIdx;

  SerialLine *sl;
  QSocketNotifier *lineNotifier;
  bool localEcho = false;
  QByteArray initstr;
  bool hangupInProgress = false;
  int logfile;

  void doXfer( XFerThread *xt, const QString& progressCaption = QString() );
  void rxFileXModem();
  void detectZModem( unsigned char c );
  void detectIEmsi( unsigned char c );
  void txFilesXModem( const QStringList& files );
  void txFilesXModem1k( const QStringList& files );
  void txFilesYModem( const QStringList& files );
  void txFilesZModem( const QStringList& files );

  void outp_at( int pos, const QChar& c );
  QChar translateChar( unsigned char c );
  void setFgColor( int color_idx );
  void process_m_cmd();
  void process_n_cmd();
  void moveCursor( int new_pos );
  void repMoveCursor( int dir );
  void process_A_cmd();
  void process_B_cmd();
  void process_C_cmd();
  void process_D_cmd();
  void process_G_cmd();
  void eraseScreenPart( int start, int end );
  void process_J_cmd();
  void eraseLinePart( int pos, int count );
  void process_K_cmd();
  void process_H_cmd();
  void process_P_cmd();
  void process_X_cmd();
  void process_d_cmd();
  void cellChanged( int cell );
  void repaintCell( int cell );
  void fixCursor();
  virtual void paintEvent( QPaintEvent *e );
  virtual void focusInEvent( QFocusEvent *e );
  virtual void focusOutEvent( QFocusEvent *e );
  virtual void keyPressEvent( QKeyEvent *e );
  virtual void mousePressEvent( QMouseEvent *e );
  virtual void mouseReleaseEvent( QMouseEvent *e );
  virtual void mouseMoveEvent( QMouseEvent *e );
  virtual void customEvent( QCustomEvent *e );
protected slots:
  void drawCursor();
  void fetchFromLine( int fd );
  void lineReopened( SerialLine *sl );
  void finishHangup();

  void processCdDrop();

  void processFileRxStart( XferEvent *e );
  void processFileRxResume( XferEvent *e );
  void processFileRxDone();

  void processFileTxStart( XferEvent *e );
  void processFileTxResume( XferEvent *e );
  void processFileTxDone();

  void processXferSkipRequest();
  void processXferStopRequest();

  void rxFileXModem( const QString& fname );
  void txFileXModem( const QString& fname );
  void queueRxXModem( const QString& fname );
  void queueTxXModem( const QString& fname );

  void grabLine();
  void releaseLine();
  void log( const QString& msg );
};

class ConfigManager {
public:
  ConfigManager();
  void mkpath( const QString& d );
  static QString getFilename( const QString& confName );
  static QString getVarPath( const QString& fname );

protected:
  static QString config, var;
};
