#include <qlabel.h>
#include <qprogressbar.h>
#include <qtimer.h>
#include <qwidgetstack.h>
#include <qpushbutton.h>
#include "moc_transferdialog.h"

TransferDialog::TransferDialog( QWidget *parent ):
	TransferUI( parent, "TransferDialog", true ),
	emitDoneEnabled(false), xferInfoTimer(0) {

  setFixedSize( size() );

  filenameL->clear();
  filesizeL->clear();
  transferredL->clear();
  cpsL->clear();
  estimatedTimeL->clear();
  elapsedTimeL->clear();

  connect( skipB, SIGNAL(clicked()), SIGNAL(skipFileRequested()) );

  show();
}

void TransferDialog::customEvent( QCustomEvent *event0 ) {
  XferEvent *e;

  switch ( event0->type() ) {
  case XFER_FILE_INFO:
	e = (XferEvent*)event0;
	filenameL->setText( e->fname );

	switch ( e->bytes ) {
	case -1:
		progressPB->setTotalSteps( 0 );
		break;
	case  0:
		progressPB->setTotalSteps( 1 );
		filesizeL->setText( QString::number( e->bytes ) );
		break;
	default:
		progressPB->setTotalSteps( e->bytes );
		filesizeL->setText( QString::number( e->bytes ) );
	}

	progressPB->setProgress( 0 );

	xferFileSize = e->bytes;
	xferStartPos = 0;
	xferTransferredBytes = 0;

	if ( !xferInfoTimer ) {
		xferInfoTimer = new QTimer( this );
		connect( xferInfoTimer, SIGNAL(timeout()), SLOT(updateXferInfo()) );
		xferInfoTimer->start( 1000 );
	}
	transferStartTime = time(0);

	emit fileTransferStart( e );
	emitDoneEnabled = true;
  	break;
  case XFER_FILE_RESUME:
  	e = (XferEvent*)event0;
	xferStartPos = e->bytes;
	xferTransferredBytes = e->bytes;
	if ( xferFileSize )
		progressPB->setProgress( xferTransferredBytes );

	emit fileTransferResume( e );
	break;
  case XFER_FILE_DATA:
	e = (XferEvent*)event0;
	xferTransferredBytes = e->bytes;
  	break;
  case XFER_FILE_DONE:
	if ( xferFileSize != 0 )
		progressPB->setProgress( xferFileSize );
	else
		progressPB->setProgress( 1 );

	if ( xferInfoTimer ) {
		delete xferInfoTimer;
		xferInfoTimer = 0;
	}

	if ( emitDoneEnabled ) emit fileTransferDone();

	emitDoneEnabled = false;
  	break;
  case XFER_SESSION_TERM:
	if ( xferInfoTimer ) {
		delete xferInfoTimer;
		xferInfoTimer = 0;
	}

	if ( emitDoneEnabled ) emit fileTransferDone();

	emitDoneEnabled = false;
  	break;
  }
}

void TransferDialog::updateXferInfo() {
  time_t ct;
  int transferSec, estimatedSec, min, sec, cps;

  transferredL->setText( QString::number( xferTransferredBytes ) );

  if ( xferFileSize != 0 )
	progressPB->setProgress( xferTransferredBytes );

  ct = time(0);
  transferSec = ct - transferStartTime;
  min = transferSec / 60;
  sec = transferSec % 60;
  elapsedTimeL->setText( QString().sprintf( "%d:%02d", min, sec ) );

  if ( !transferSec ) cps = xferTransferredBytes - xferStartPos;
  else cps = (xferTransferredBytes - xferStartPos) / transferSec;
  cpsL->setText( QString::number( cps ) );
  
  if ( cps && xferFileSize > 0 ) {
	estimatedSec = (xferFileSize - xferStartPos) / cps;
	if ( estimatedSec < transferSec ) estimatedSec = transferSec;
	min = estimatedSec / 60;
	sec = estimatedSec % 60;
	estimatedTimeL->setText( QString().sprintf( "%d:%02d", min, sec  ) );
  }
  else
  	estimatedTimeL->clear();
}

void TransferDialog::cancelXfer() {
  progressWS->raiseWidget( 1 );
  skipB->setEnabled( false );
  cancelB->setEnabled( false );

  emit cancelSessionRequested();
}
